import { mount, createLocalVue } from "@vue/test-utils";
import Vuex from "vuex";
import Iframe from "../../../src/components/LayoutBuilderIframe.vue";
import state from "../../mocks/state";
import Store from "../../../src/store/index";

const localVue = createLocalVue();
const store = Store(state);

localVue.use(Vuex);

describe("Iframe.vue", () => {
  describe("state.iframeSrc is null", () => {
    describe("isHidden()", () => {
      const wrapper = mount(Iframe, { store: Store(state), localVue });

      it("returns true is iframeSrc is null", () => {
        expect(wrapper.vm.isHidden).toEqual(true);
      });
    });
  });

  describe("state.iframeSrc is not null", () => {
    const iframeSrc = "foo";
    const wrapper = mount(Iframe, {
      store: Store(Object.assign({}, state, { iframeSrc })),
      localVue
    });

    describe("iframeSrc()", () => {
      it("returns iframeSrc from the state", () => {
        expect(wrapper.vm.iframeSrc).toEqual(iframeSrc);
      });
    });

    describe("isHidden()", () => {
      it("returns false", () => {
        expect(wrapper.vm.isHidden).toEqual(false);
      });
    });
  });

  describe("cancel()", () => {
    const wrapper = mount(Iframe, { store, localVue });
    const stub = jest.fn();
    store.commit = stub;

    it("commits setIframeSrc with null to the store", () => {
      wrapper.vm.cancel();
      expect(stub).toHaveBeenCalledWith("setIframeSrc", null);
    });

    it("commits setCurrentlyEditableContent with null to the store", () => {
      wrapper.vm.cancel();
      expect(stub).toHaveBeenCalledWith("setCurrentlyEditableContent", null);
    });
  });

  describe("clicking on the overlay", () => {
    const wrapper = mount(Iframe, {
      store,
      localVue
    });
    const stub = jest.fn();

    wrapper.setMethods({ onBackgroundClick: stub });

    it("calls onBackgroundClick()", () => {
      wrapper.trigger("click");
      expect(stub).toHaveBeenCalled();
    });
  });

  describe("calling onBackgroundClick", () => {
    describe("with the clicked event being the root element", () => {
      const wrapper = mount(Iframe, {
        store,
        localVue
      });
      const stub = jest.fn();

      wrapper.setMethods({ cancel: stub });

      it("calls cancel()", () => {
        wrapper.vm.onBackgroundClick({
          target: wrapper.element
        });

        expect(stub).toHaveBeenCalled();
      });
    });

    describe("with the clicked event not being the root element", () => {
      const wrapper = mount(Iframe, {
        store,
        localVue
      });
      const stub = jest.fn();

      wrapper.setMethods({ cancel: stub });

      it("does not call cancel()", () => {
        wrapper.vm.onBackgroundClick({
          target: wrapper.element.firstChild
        });

        expect(stub).not.toHaveBeenCalled();
      });
    });
  });

  describe("calling onKeyDown", () => {
    describe("with the keyCode being 27", () => {
      const wrapper = mount(Iframe, {
        store,
        localVue
      });
      const stub = jest.fn();

      wrapper.setMethods({ cancel: stub });

      it("calls cancel()", () => {
        wrapper.vm.onKeyDown({
          keyCode: 27
        });

        expect(stub).toHaveBeenCalled();
      });
    });

    describe("with the keyCode not being 27", () => {
      const wrapper = mount(Iframe, {
        store,
        localVue
      });
      const stub = jest.fn();

      wrapper.setMethods({ cancel: stub });

      it("does not call cancel()", () => {
        wrapper.vm.onKeyDown({
          keyCode: 13
        });

        expect(stub).not.toHaveBeenCalled();
      });
    });
  });

  describe("firing contentEdited on the iframe", () => {
    const detail = {
      id: 1,
      element: {
        html: "foo",
        editUrl: "bar"
      }
    };
    const wrapper = mount(Iframe, {
      store,
      localVue
    });
    const stub = jest.fn();
    store.dispatch = stub;

    it("dispatches editContent on the store with the correct params", () => {
      wrapper.element.querySelector("iframe").dispatchEvent(
        new CustomEvent("contentEdited", {
          detail
        })
      );
      expect(stub).toHaveBeenCalledWith("editContent", detail);
    });
  });
});
